#include "location.h"

/*#DOC  Title:																*/
/*#DOC    calc_delaz														*/
/*#DOC  Desc:																*/
/*#DOC    Fills 'delta' and 'esaz' fields for each phase.					*/
/*#DOC  Input Arguments:													*/
/*#DOC    sp  - pointer to structure containing current solution.			*/
/*#DOC    p[] - array of pointers to structures containing phases.			*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/
/*#DOC  Calls:																*/
/*#DOC	  calc_delta() and calc_esaz() from utils.c.						*/
/*#DOC    print_pha() from utils.c											*/

int calc_delaz (struct sol_rec *sp, struct pha_rec p[])
{
	double delta,esaz;
	int i;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	for (i=0; i<sp->numphas; i++){

		/* Assume that readings come together with initial phase first. */
		/* Then only have to calculate delta and azimuth once per reading. */
		if (p[i].init){
			delta=calc_delta(p[i].dircos,sp->dircos);
			esaz =calc_esaz(p[i].dircos,sp->dircos,p[i].sta_lon,sp->lon,delta);
		}
		p[i].delta = delta;
		p[i].esaz  = esaz;
	}
	if (diagnostic){
		printf ("calc_delaz done:\n");
		print_pha(sp->numphas, p);
	}
	return 0;
}
