#include "location.h"

/*#DOC  Title:																*/
/*#DOC    calc_depdp														*/
/*#DOC  Desc:																*/
/*#DOC    Calls a sequence of functions to identify pP phases and use		*/
/*#DOC    them to calculate a depth phase depth.							*/
/*#DOC  Input Arguments:													*/
/*#DOC    sp - pointer to structure containing current solution.			*/
/*#DOC    p  - array of phase structures.									*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/
/*#DOC  Calls:																*/
/*#DOC	  find_pP(), pP_duplicates(), calc_pP_resid(), solve_depdp()		*/

/* id_pha must have been called before this function.						*/

int calc_depdp (struct sol_rec *sp, struct pha_rec p[])
{

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* Search for unidentified pP phases. */
	if (diagnostic)
		fprintf (logfp,"Calling find_pP\n");
	if ( find_pP(sp,p) )
		return 1;

	/* Deal with duplicate pP phases. */
	if (diagnostic)
		fprintf (logfp,"Calling pP_duplicates\n");
	if ( pP_duplicates(sp,p) )
		return 1;

	/* Set pP_resid and pP_dtdh for any pP phases. */
	if (diagnostic)
		fprintf (logfp,"Calling calc_pP_resid\n");
	if ( calc_pP_resid(sp,p) )
		return 1;

	/* Calculate depth phase depth if possible. */
	if (diagnostic)
		fprintf (logfp,"Calling solve_depdp\n");
	if ( solve_depdp(sp,p) )
		return 1;
	
	return 0;
}
