#include "location.h"
#include "jb_model.h"

/*#DOC  Title:																*/
/*#DOC    calc_elev_corr													*/
/*#DOC  Desc:																*/
/*#DOC    Calculates travel time correction for elevation of station.		*/
/*#DOC  Input Arguments:													*/
/*#DOC    pp - pointer to phase structure.									*/
/*#DOC  Return:																*/
/*#DOC    Travel time correction for station elevation.						*/

/* If both an elevation correction and a crustal correction are required	*/
/* then the elevation correction is done in calc_crust_corr, not here.		*/

double calc_elev_corr(struct pha_rec *pp)
{
	double elev_corr;

	/* Not all stations have a known elevation. */
	if (pp->sta_elev == NULLVAL)
		return 0;

	/* Do P and S arrivals but not Pg or Sg. */
	if (strcmp(pp->phase,"P") ==0  || 
		strcmp(pp->phase,"PKP")==0 ||
		strcmp(pp->phase,"Pn")==0  ||
		strcmp(pp->phase,"Pb")==0){

		elev_corr  = PGVEL*(pp->dtdd/KM_PER_DEG);
		elev_corr *= elev_corr;
		elev_corr  = 1 / sqrt( 1 - elev_corr);
		elev_corr *= pp->sta_elev/(1000*PGVEL);
	}
	else if (strcmp(pp->phase,"S")==0  || 
			strcmp(pp->phase,"SKS")==0 ||
			strcmp(pp->phase,"Sn")==0  ||
			strcmp(pp->phase,"Sb")==0){

		elev_corr  = SGVEL*(pp->dtdd/KM_PER_DEG);
		elev_corr *= elev_corr;
		elev_corr  = 1 / sqrt( 1 - elev_corr);
		elev_corr *= pp->sta_elev/(1000*SGVEL);
	}
	else
		elev_corr = 0;

	return elev_corr;
}
