#include "location.h"

/* Steps in delta used to get multiplier for geoid correction. */
int delta_step[] = { 9, 13, 22, 41, 63, 72, 83, 124, 140, 180 };

/*#DOC  Title:																*/
/*#DOC    calc_geoid_corr													*/
/*#DOC  Desc:																*/
/*#DOC    Calculates travel time correction for height above geoid.			*/
/*#DOC  Input Arguments:													*/
/*#DOC    lat - source latitude.											*/
/*#DOC    pp  - pointer to  a phase structure.								*/
/*#DOC  Return:																*/
/*#DOC    Travel time correction for height of geoid.						*/
/*#DOC  Calls:																*/
/*#DOC    height_above_mean_sphere() from this file.						*/

double calc_geoid_corr(double lat, struct pha_rec *pp)
{
	double height_above_mean_sphere(double lat);
	double epi_hams,sta_hams;
	double elcor,geoid_corr;
	int i;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* Apply corrections at station and epicentre together.	*/
	/* delta_step array defined at top of file.				*/
	epi_hams = height_above_mean_sphere(lat*TORAD);
	sta_hams = height_above_mean_sphere(pp->sta_lat*TORAD);

	if (strcmp(pp->phase,"PKP")==0){
		if (pp->delta < 140)
			elcor = 0.1;
		else
			elcor = 0.094;
	}
	else{
		for (i=0; (int)(pp->delta+0.5) > delta_step[i]; i++);
		elcor = 0.01 * i;
	}
	geoid_corr = elcor * (epi_hams+sta_hams);

	return geoid_corr;
}


/*#DOC  Title:																*/
/*#DOC    height_above_mean_sphere	(only called by calc_geoid_corr)		*/
/*#DOC  Desc:																*/
/*#DOC    Calculates height above mean sphere.								*/
/*#DOC  Input Arguments:													*/
/*#DOC    lat  - latitude in radians.										*/
/*#DOC  Return:																*/
/*#DOC    Height above mean sphere in km.									*/

double height_above_mean_sphere(double lat)
{
	return 10.738*cos(2*lat)-3.549-0.023*cos(4*lat);
}
