#include "location.h"
#include "bodymag_cor.h"

/*#DOC  Title:																*/
/*#DOC    calc_sta_mb														*/
/*#DOC  Desc:																*/
/*#DOC    Fill bodymag field for phase if it has suitable amplitudes.		*/
/*#DOC  Input Arguments:													*/
/*#DOC    sp - pointer to structure containing current solution.			*/
/*#DOC    pp  - pointer to a phase structure.								*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/

int calc_sta_mb (struct sol_rec *sp, struct pha_rec *pp)
{
	extern double body_mag_min_dist;		/* From config file */
	extern double body_mag_max_dist;		/* From config file */
	extern double body_mag_min_per;			/* From config file */
	extern double body_mag_max_per;			/* From config file */

	double logat;
	double minper;
	double depth_erad;
	int row,col;
	int totlogat,numlogat;
	int i;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* Only calculate for P phases within given distance range. */
	if (strcmp(pp->phase,"P") == 0 && pp->delta >= body_mag_min_dist
        && pp->delta <= body_mag_max_dist && pp->numamps > 0){

		/* If more than one amp then use one with shortest period. */
		minper = body_mag_max_per;
		logat = NULLVAL;
		for (i=0; i < pp->numamps; i++)
			if (pp->a[i].per >= body_mag_min_per && pp->a[i].per <= minper){
				if (pp->a[i].amp == 0 || pp->a[i].amp == NULLVAL)
					continue;
				minper = pp->a[i].per;
				logat = log10(pp->a[i].amp/pp->a[i].per);
			}

		/* If no amp/per pairs then average any log(a/t) values. */
		if (logat == NULLVAL){
			totlogat = numlogat = 0;
			for (i=0; i < pp->numamps; i++)
				if (pp->a[i].logat != NULLVAL){
					totlogat += pp->a[i].logat;
					numlogat++;
				}
			if (numlogat)
				logat = totlogat/numlogat;
		}

		/* Should now have single logat for this phase. (unless amp=0 in db) */
		if (logat != NULLVAL){
			depth_erad = (sp->depth - MOHO)/MOHO_RADIUS;
			col = 1.5 + 100*depth_erad;
			row = pp->delta+0.5;

			pp->bodymag = logat + (0.1*magcor[col][row]) - 3;

			if (diagnostic){
				fprintf(logfp,"calc_sta_mb: depth=%.2f,col=%d ",sp->depth,col);
				fprintf(logfp,"%s %s ",pp->sta,pp->phase);
				fprintf(logfp,"logat=%.2f, magcor=%d ",logat,magcor[col][row]);
				fprintf(logfp,"bodymag=%.2f\n",pp->bodymag);
			}
		}
	}
	return 0;
}
