#include "location.h"

/*#DOC  Title:																*/
/*#DOC    calc_weight														*/
/*#DOC  Desc:																*/
/*#DOC    Calls the function corresponding to current weighting_type.		*/
/*#DOC  Input Arguments:													*/
/*#DOC    sp - pointer to structure containing current solution.			*/
/*#DOC    p[]  - array of structures containing phases.						*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/
/*#DOC  Calls:																*/
/*#DOC    function to carry out weighting.									*/
/*#DOC    print_pha() from utils.c											*/

/* This function is run each iteration. It needs to be run after:			*/
/*		get_weight_factor													*/
/*		mark_duplicates														*/
/*		calc_resid															*/

int calc_weight (struct sol_rec *sp, struct pha_rec p[])
{
	int status,i;
	double total_weight;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* Alpha is calculated in weighting routine.						*/
	/* For first iteration prev_alpha also set by weighting routine.	*/
	if (sp->iteration != 0)
		sp->prev_alpha = sp->alpha;

	/* Values of weighting_type correspond to possible entries for		*/
	/* weighting1 and weighting2 in config.txt							*/
	if (strcmp(sp->weighting_type,"huber")==0){
		if (huber_weight(sp,p))
			return 1;
	}
	else if (strcmp(sp->weighting_type,"buland")==0){
		if (buland_weight(sp,p))
			return 1;
	}
	else if (strcmp(sp->weighting_type,"none")==0){
		if (no_weight(sp,p))
			return 1;
	}

	/* Update sigma, only for use by decide_iter if huber/buland weighting.	*/
	/* sigma = sqrt ( sum simulb**2 / sum weight ). 						*/
	sp->prev_sigma = sp->sigma;
	sp->sigma = 0;
	total_weight = 0;
	for (i=0; i< sp->numphas; i++){
		sp->sigma += p[i].resid*p[i].resid*p[i].weight;
		total_weight += p[i].weight;
	}
	sp->sigma = sqrt(sp->sigma/total_weight);

	if (diagnostic){
		fprintf(logfp,"calc_weight: %s:  ",sp->weighting_type);
		fprintf(logfp,"sigma=%f alpha=%f\n",sp->sigma,sp->alpha);
		print_pha(sp->numphas, p);
	}
	return 0;
}
