#include "location.h"

/*#DOC  Title:																*/
/*#DOC    correct_ttime														*/
/*#DOC  Desc:																*/
/*#DOC    Potentialy changes 'ttime' for a phase.							*/
/*#DOC  Input Arguments:													*/
/*#DOC    sp - pointer to structure containing current solution.			*/
/*#DOC    pp - pointer to  a phase structure.								*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/
/*#DOC  Calls:																*/
/*#DOC    calc_geoid_corr													*/
/*#DOC    calc_elev_corr if elev_corr_on = 1 and crust_corr_on = 0			*/					/*#DOC    calc_crust_corr if crust_corr_on = 1								*/

/* Crustal correction dodgy for small delta - take off angles too big.		*/
/* Set the smallest delta that a correction will be calculated for here.	*/
#define CRUST_CORR_MIN_DELTA 20

int correct_ttime (struct sol_rec *sp, struct pha_rec *pp)
{
	extern int crust_corr_on;	/* From config.txt. */
	extern int elev_corr_on;	/* From config.txt. */
	extern int mantle_time_on;	/* From config.txt. */

	double geoid_corr,source_corr,station_corr;
	double elev;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* Apply travel time correction at station and 		*/
	/* epicentre for height of geoid above mean sphere.	*/
	geoid_corr = calc_geoid_corr(sp->lat, pp);
	pp->ttime += geoid_corr;

	if (diagnostic)
		fprintf(logfp,"correct_ttime: %s geoid_corr=%.3f\n",pp->sta,geoid_corr);

	/* Check whether crustal correction is switched on in config file. */
	if (crust_corr_on && pp->delta > CRUST_CORR_MIN_DELTA){

		/* P */
		if (strcmp(pp->phase,"P") == 0){

			/* No elevation correction for source. */
			elev = NULLVAL;

			/* Source end correction. */
			source_corr=calc_crust_corr('P',sp->lat,sp->lon,sp->depth,
															elev,pp->dtdd);

			/* To not calculate elevation correction set elev = NULLVAL. */
			if (elev_corr_on)
				elev = pp->sta_elev;
			else
				elev = NULLVAL;

			/* Station end correction. */
			station_corr=calc_crust_corr('P',pp->sta_lat,pp->sta_lon,0,
															elev,pp->dtdd);

			if (diagnostic){
				fprintf(logfp,"correct_ttime:P: %s ",pp->sta);
				fprintf(logfp,"source_corr=%.3f ",source_corr);
				fprintf(logfp,"station_corr=%.3f\n",station_corr);
			}
			pp->ttime += source_corr + station_corr;
		}
		/* S */
		else if (strcmp(pp->phase,"S") == 0){

			/* No elevation correction for source. */
			elev = NULLVAL;

			/* Source end correction. */
			source_corr=calc_crust_corr('S',sp->lat,sp->lon,sp->depth,
															elev,pp->dtdd);

			if (diagnostic)
				fprintf(logfp,"correct_ttime:S: %s %.3f\n",pp->sta,pp->delta);


			/* To not calculate elevation correction set elev = NULLVAL. */
			if (elev_corr_on)
				elev = pp->sta_elev;

			/* Station end correction. */
			station_corr=calc_crust_corr('S',pp->sta_lat,pp->sta_lon,0,
															elev,pp->dtdd);

			if (diagnostic){
				fprintf(logfp,"correct_ttime:S: %s ",pp->sta);
				fprintf(logfp,"source_corr=%.3f ",source_corr);
				fprintf(logfp,"station_corr=%.3f\n",station_corr);
			}
			pp->ttime += source_corr + station_corr;
		}
	}
	/* Elevation correction already checked if crust correction done. */
	else if (elev_corr_on){
		station_corr = calc_elev_corr(pp);
		pp->ttime += station_corr;

		if (diagnostic){
			fprintf(logfp,"correct_ttime: %s ",pp->sta);
			fprintf(logfp,"elev_corr=%.3f\n",station_corr);
		}
	}
	return 0;
}
