#include "location.h"

/*#DOC  Title:																*/
/*#DOC    decide_iter														*/
/*#DOC  Desc:																*/
/*#DOC    Tests for convergence/divergence of current solution.				*/
/*#DOC    If found updates the corresponding flag in the solution structure.*/
/*#DOC  Input Arguments:													*/
/*#DOC    sp - pointer to structure containing current solution.			*/
/*#DOC    p[] - array of phase structures.									*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/

int decide_iter (struct sol_rec *sp, struct pha_rec p[])
{
	extern double avg_weight_thresh;	/* From config file */
	extern double alpha_thresh;			/* From config file */
	extern double dalpha_thresh;		/* From config file */
	extern double dsigma_low;			/* From config file */
	extern double dsigma_high;			/* From config file */

	int ndef,i;
	double avg_weight;
	double dsigma;
	double dalpha;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* Calculate average weight */
	avg_weight =0;
	ndef=0;
	for (i=0; i<sp->numphas; i++){
		if (p[i].weight_factor > 0){
			avg_weight += p[i].weight;
			ndef++;
		}
	}
	avg_weight /= ndef;

	/* Calculate changes in alpha and sigma since last iteration */
	dsigma = fabs(sp->sigma - sp->prev_sigma);
	dalpha = fabs(sp->alpha - sp->prev_alpha);

	if (diagnostic){
		fprintf(logfp,"decide: avg=%f ndef=%d ",avg_weight,ndef);
		fprintf(logfp,"alpha=%f prev_alpha=%f\n",sp->alpha,sp->prev_alpha);
	}

	/* Check for convergence */
	if ( fabs(sp->alpha) <= alpha_thresh 	&& 
		 dsigma > dsigma_low				&&
		 dsigma < dsigma_high				&&
		 avg_weight > avg_weight_thresh ){

		sp->converged = 1;

		if (diagnostic){
			fprintf(logfp,"decide: converged: avg=%f da=%f ",avg_weight,dalpha);
			fprintf(logfp,"ds=%f depth=%f\n",dsigma,sp->depth);
		}
		return 0;
	}

	/* Check for divergence */
	if (avg_weight < avg_weight_thresh
	  || (dalpha > dalpha_thresh && dsigma < dsigma_low)){
			
		sp->diverging = 1;

		if (diagnostic){
			fprintf(logfp,"decide: diverging: avg=%f da=%f ",avg_weight,dalpha);
			fprintf(logfp,"ds=%f depth=%f\n",dsigma,sp->depth);
		}
	}
	return 0;
}
