#include "location.h"

#define NUM_MAX_RESID 3				 /* Must match structure below. */

struct pP_max_reisd_rec {
	double depth;
	double max_resid;
};

struct pP_max_reisd_rec pP_max_reisd [NUM_MAX_RESID] = {
	{ 0, 2},
	{33, 3},
	{44, 5},
};


/*#DOC  Title:																*/
/*#DOC    find_pP															*/
/*#DOC  Desc:																*/
/*#DOC    Checks phases without ISC id to see if they're pP.				*/
/*#DOC  Input Arguments:													*/
/*#DOC    sp - pointer to structure containing current solution.			*/
/*#DOC    p  - array of structures containing phases.						*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/
/*#DOC  Uses:																*/
/*#DOC	  read_pP_P() from read_pP_P.c										*/
/*#DOC	  add_to_error() from utils.c.										*/

/* This function must be called after id_pha.	*/

int find_pP (struct sol_rec *sp, struct pha_rec p[])
{
	double p_time;
	double threshold;
	char error_part[ERRLEN];
	int p_index  = 0;
	int i,j;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* Can't calculate residuals unless have a depth for the source. */
	if (sp->depth == NULLVAL){
		add_to_error("calc_pP_resid: depthless hypocentre.");
		return 1;
	}

	/* Loop through all associated phases. */
	for (i=0; i<sp->numphas; i++){

		/* Timeless phases are of no interest. */
		if (p[i].time == NULLVAL)
			continue;

		/* For each reading store P time.		*/
		/* There can only be one P per reading.	*/
		if (strcmp(p[i].phase,"P") == 0){
			p_time = p[i].time;
			p_index   = i;
		}

		/* Check all unidentified phases in case they're pP.	*/
		if (!p[i].phase[0] || strcmp(p[i].phase,"pP")==0 ){

			/* Check that a P time was stored is for this reading. */
			if (p[i].rdid != p[p_index].rdid){
				strcpy(p[i].phase,"");
				continue;
			}
			/* Get pP-P time and corresponding dtdh from tables. 	*/
			/* If delta/depth out of range will return unwanted		*/
			/* error - clear error string and skip to next phase.	*/
			if ( read_pP_P(sp->depth, &p[i]) ){
				strcpy(p[i].phase,"");
				strcpy(errstr,"");
				continue;
			}

			/* If reporter called it pP then that's enough. */
			/*if(strcmp(p[i].phase,"pP")==0)
				continue;*/

			/* Check residual to see if it's small enough to be pP. */
			p[i].pP_resid = p[i].time - p_time - p[i].pP_P_time;

			if (diagnostic){
				fprintf(logfp,"find_pP: %5s %2s ",p[i].sta,p[i].phase);
				fprintf(logfp,"%0.3f r=%f\n",p[i].delta,p[i].pP_resid);
			}

			/* Find residual threshold for this depth from structure at top. */
			for (j=0;j<NUM_MAX_RESID;j++)
				if (sp->depth > pP_max_reisd[j].depth)
					threshold = pP_max_reisd[j].max_resid;

			if (fabs(p[i].pP_resid) > threshold){
				strcpy(p[i].phase,"");
				p[i].pP_resid  = NULLVAL;
				p[i].pP_P_time = NULLVAL;
				p[i].pP_dtdh   = NULLVAL;
				continue;
			}
			strcpy(p[i].phase,"pP");

			if (diagnostic)
				fprintf(logfp,"find_pP: %s identified as pP\n",p[i].rep_phase);
		}
	}
	return 0;
}
