#include "location.h"

/*#DOC  Title:																*/
/*#DOC    get_data															*/
/*#DOC  Desc:																*/
/*#DOC    Reads database and populates hypocentre and phase structures.		*/
/*#DOC    Assigns memory dynamicaly for the arrays of structures needed.	*/
/*#DOC  Input Arguments:													*/
/*#DOC    ep - pointer to structure containing event information.			*/
/*#DOC    hp - pointer to the address of the hypocentre structure array.	*/
/*#DOC    pp - pointer to the address of the phase structure array.			*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/
/*#DOC  Calls:																*/
/*#DOC	  get_event(), get_hyp(), and get_pha()								*/

/* It is neceassary to pass pointers to addresses for the arrays of 		*/
/* structures because calloc assigns a value to these addresses when it		*/
/* finds space for the structure arrays.									*/

int get_data(struct event_rec *ep, struct hyp_rec *hp[], struct pha_rec *pp[])
{

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* Get prime, numhyps and numphas. */
	if (get_event(ep)){
		return 1;
	}

	if (diagnostic){
		fprintf(logfp,"get_data: evid=%d numhyps=%d\n",ep->evid,ep->numhyps);
		fprintf(logfp,"get_data: prime=%d numphas=%d\n",ep->prime,ep->numphas);
	}

	/* Assign memory to structure arrays (via pointers so can pass back). */
	*hp = (struct hyp_rec *) calloc (ep->numhyps, sizeof(struct hyp_rec));
	if (*hp == NULL){
		sprintf(errstr,"get_data: evid %d: hyp memory problem.",ep->evid);
		return 1;
	}

	*pp = (struct pha_rec *) calloc (ep->numphas, sizeof(struct pha_rec));
	if (*pp == NULL){
		sprintf(errstr,"get_data: evid %d: pha memory problem.",ep->evid);
		free(*hp);
		return 1;
	}

	/* Fill array of hypocenter structures. */
	if (get_hyp(ep,*hp)){
		free(*hp);
		free(*pp);
		return 1;
	}

	/* Fill array of phase structures. */
	if (get_pha(ep,*pp)){
		free(*hp);
		free(*pp);
		return 1;
	}
	return 0;
}
