#include "location.h"

/*#DOC  Title:																*/
/*#DOC    get_weight_factor													*/
/*#DOC  Desc:																*/
/*#DOC    Fills 'weight_factor' field for each phase.						*/
/*#DOC  Input Arguments:													*/
/*#DOC    p[] - array structures containing phases.					 		*/
/*#DOC    numphas  - number of phases in p[].								*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/
/*#DOC  Uses:																*/
/*#DOC    print_pha() from utils.c											*/

#define NUM_PHASE_WEIGHTS 12				 /* Must match structure below. */

struct phase_weight_rec {
	char phase[9];
	int delta1;
	int delta2;
	double weight_factor;
};

struct phase_weight_rec phase_weight[NUM_PHASE_WEIGHTS] = {
	{ "P",  0, 20, 1 },
	{ "P",  20, 100, 1 },
	{ "Pn", 0,   8, 1 },
	{ "Pg", 0,   8, 1 },
	{ "Pb", 0,   8, 1 },
	{ "PKP", 110, 180, 0 },
	{ "S" , 0, 10, 1 },
	{ "S" ,10,20, 1 },
	{ "S" ,20,65, 0.5 },
	{ "Sn", 0,10, 1 },
	{ "Sg", 0,10, 1 },
	{ "Sb", 0,10, 1 },
};

/* The weight for each phase is multiplied by weight_factor before use.		*/
/* In addition, weight_factor is used as a switch (0 or not) for deciding	*/
/* whether to calculate phase residuals in the main iteration loop and		*/
/* whether to include a phase when calculating average weights etc.			*/
/* The factor is decided depending on phase code and delta using the		*/
/* numbers stored in the phase_weight structure. In addition phases			*/
/* without arrival times get weight_factor 0 and so do phases that have		*/
/* been purged in a previous iteration.										*/

/* This function is run each iteration. It needs to be run after:			*/
/*		calc_delaz															*/
/*		id_pha																*/

int get_weight_factor (struct pha_rec p[], int numphas)
{
	int i,j;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	for (i=0; i<numphas; i++){

		p[i].weight_factor = 0;

		/* Purged phases get a weight factor of 0.	*/
		if (p[i].purged)
			continue;

		/* No point giving a timeless phase a weight.	*/
		if (p[i].time == NULLVAL)
			continue;

		/* Use the phase_weight structure to decide weight_factor.	*/
		/* for this phases phase code and delta.					*/
		for (j=0;j<NUM_PHASE_WEIGHTS;j++){
			if (strcmp(p[i].phase,phase_weight[j].phase)==0){
				if (p[i].delta > phase_weight[j].delta1 &&
					p[i].delta <= phase_weight[j].delta2	)
						p[i].weight_factor = phase_weight[j].weight_factor;
				else
					if (strcmp(p[i].phase,phase_weight[j+1].phase)!=0)
						break;
			}
		}
	}
	if (diagnostic){
		fprintf(logfp,"get_weight_factor done:\n");
		print_pha(numphas, p);
	}
	return 0;
}
