#include "location.h"

/*#DOC  Title:																*/
/*#DOC    init_sol															*/
/*#DOC  Desc:																*/
/*#DOC    Sets up solution structure with starting point for 1st iteration.	*/
/*#DOC  Input Arguments:													*/
/*#DOC    sp  - pointer to solution structure to be initialised.			*/
/*#DOC    ep  - pointer to structure containing event information.			*/
/*#DOC    hp  - pointer to structure containing seed hypocentre.			*/
/*#DOC  Calls:																*/
/*#DOC    calc_dircos() from utils.c										*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/

int init_sol(struct sol_rec *sp, struct event_rec *ep, struct hyp_rec *hp)
{
	extern double sigma_start;		/* From config file */
	extern char weighting1[];		/* From config file */
	extern double default_depth;	/* From config file */
	int i;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* Mark as not converged yet. */
	sp->converged = 0;
	sp->diverging = 0;

	/* numphas stored in both event and solution structures for convenience. */
	sp->numphas = ep->numphas;

	/* Set starting point for location to that of chosen hypocentre */
	/* or to that fixed for the event. */
	if (ep->fixed_lat != NULLVAL){
		sp->lat = ep->fixed_lat;
		sp->lon = ep->fixed_lon;

		calc_dircos(sp->lat,sp->lon,sp->dircos);
	}
	else {
		sp->lat  = hp->lat;
		sp->lon  = hp->lon;

		for (i=1; i<=6; i++)
			sp->dircos[i] = hp->dircos[i];
	}

	/* Depth either from hypocentre or that fixed for the event. */
	if (ep->fixed_depth != NULLVAL){
		sp->depth = ep->fixed_depth;
	}
	else {
		if (hp->depth != NULLVAL)
			sp->depth = hp->depth;
		else
			sp->depth = default_depth;
	}

	/* Time either from hypocentre or that fixed for the event. */
	if (ep->fixed_time != NULLVAL)
		sp->time = ep->fixed_time;
	else
		sp->time = hp->time;

	/* Reset errors and sdobs. */
	/* Only for diagnostics - they get overwritten anyway. */
	for (i=0;i<5;i++)
		sp->error[i] = NULLVAL;
	sp->sdobs = NULLVAL;
	sp->mindist = sp->maxdist = NULLVAL;
	sp->majax = sp->minax = sp->theta = NULLVAL;

	/* Set starting point for weighting variables to those set in config file */
	strcpy(sp->weighting_type,weighting1);
	if (sp->weighting_type[0] == '\0'){
		sprintf(errstr,"init_sol: weighting1 not set");
		return 1;
	}
	if (sigma_start)
		sp->sigma = sigma_start;
	else{
		sprintf(errstr,"init_sol: sigma_start not set");
		return 1;
	}
	if (diagnostic){
		fprintf(logfp,"init_sol: weight=%s ",sp->weighting_type);
		fprintf(logfp,"sigma=%.3f\n",sp->sigma);
	}

	/* Depth-phase depth has not been calculated yet. */
	sp->depdp = NULLVAL;

	/* If phase purging has been disabled in instructions then set	 	*/
	/* phases_purged flag as if purging has already been done.			*/
	if (ep->purge_phase)
		sp->phases_purged = 0;
	else
		sp->phases_purged = 1;

	return 0;
}
