#include "location.h"

#define CONFIG_FILE "./config.txt"

main()
{
	extern int max_iter;						/* From config file */
	extern char instructfile[];					/* From config file */
	extern char logfile[];						/* From config file */
	extern char errfile[];						/* From config file */
	extern char isf_outfile[];					/* From config file */
	extern int update_db;						/* From config file */

	FILE *instructfp;
	FILE *isfin,*isfout;
	char instruction[LINLEN];
	char isf_infile[FILENAMELEN];
	char prev_isf_infile[FILENAMELEN];
	char timestr[24];
	struct event_rec e;
	struct hyp_rec *h;
	struct pha_rec *p;
	struct sol_rec s;
	int option,hyp;
	int total,opt0,opt1,opt2,fail;
	int database;
	int i;

	/* Switch debug messages from main on/off (1/0). */
	int diagnostic = 0;

	/* Read configuration file and assign values to	*/
	/* corresponding external variables.			*/
	if ( read_config(CONFIG_FILE) ){
		fprintf(stderr,"ERROR: %s\n",errstr);	/* logfile not open yet. */
		exit(1);
	}

	/* Open file for error messages - could want stdout/stderr. */
	if (errfile[0]){

		/* Errors get writen to logfile anyway.
		if (strcmp(logfile,errfile) == 0)
			errfp = "/dev/null";*/

		if (strcmp(errfile,"stdout")==0)
			errfp = stdout;
		else if (strcmp(errfile,"stderr")==0)
			errfp = stderr;
		else if ((errfp = fopen(errfile,"w")) == NULL){
			fprintf(stderr,"ERROR: can't open error file\n");
			exit(1);
		}
	}
	else{
		fprintf(stderr,"ERROR: no error file given\n");
		exit(1);
	}

	/* Open file for log of actions - could want stdout/stderr. */
	if (logfile[0]){
		if (strcmp(logfile,"stdout")==0)
			logfp = stdout;
		else if (strcmp(logfile,"stderr")==0)
			logfp = stderr;
		else if ((logfp = fopen(logfile,"w")) == NULL){
			fprintf(stderr,"ERROR: can't open log file\n");
			exit(1);
		}
	}
	else{
		fprintf(stderr,"ERROR: no log file given\n");
		exit(1);
	}

	/* Open file for instructions - normally stdin. */
	if (instructfile[0]){
		if (strcmp(instructfile,"stdin")==0)
			instructfp = stdin;
		else if ((instructfp = fopen(instructfile,"r")) == NULL){
			fprintf(stderr,"ERROR: can't open instruction file\n");
			exit(1);
		}
	}
	else{
		fprintf(stderr,"ERROR: no log file given\n");
		exit(1);
	}

	/* Open file for ISF output if desired - could want stdout. */
	if (isf_outfile[0]){
		if (strcmp(isf_outfile,"stdout")==0)
			isfout = stdout;
		else if ((isfout = fopen(isf_outfile,"w")) == NULL){
			fprintf(stderr,"ERROR: %s\n",errstr);	/* No point using logfile.*/
			exit(1);
		}
	}

	database=0;						/* Flag to show connection to database. */
	prev_isf_infile[0]='\0';		/* Used if data read from ISF file.		*/
	total=opt0=opt1=opt2=fail=0;	/* Counters for performance message.	*/

	/* Read next event instruction line from standard input */
	while ( fgets(instruction,LINLEN,instructfp) ){

		/* write event devider to log file. */
		fprintf(logfp,"\n=============================================\n");

		/* Either get evid for next event to be read from database	*/
		/* or filename to read next isf format event from. 			*/
		if (read_instruction(instruction,&e,isf_infile) ){
			handle_error();
			continue;
		}

		/* Read data for new event. */
		/* Assign memory for the arrays of structures to hold them.	 */
		if (e.evid != NULLVAL){
			#ifndef NO_ORACLE
				if (!database){
					if (oracle_conn()){
						handle_error();
						continue;
					}
					database=1;
				}
				/* Read hypocenters, phases, and associations from database. */
				if (diagnostic)
					fprintf (logfp,"Calling get_data\n");
				if ( get_data(&e, &h, &p) ){
					handle_error();
					continue;
				}
			#else
				fprintf (logfp,"ORACLE database not present\n");
			#endif
		}
		else{
			/* Only open isfin if different to last event's.	*/
			/* Otherwise never get to second event in file 		*/
			if (strcmp(prev_isf_infile,isf_infile)){
				if ((isfin = fopen(isf_infile,"r")) == NULL){
					sprintf(errstr,"Can't open %s\n",isf_infile);
					handle_error();
					continue;
    			}
				strcpy(prev_isf_infile,isf_infile);
			}
			if (diagnostic)
				fprintf (logfp,"Calling read_isf_event\n");
			if ( read_isf_event(isfin,&e, &h, &p) ){
				handle_error();
				continue;
			}
		}

		/* Deal with instructions. Set event_type. Check for too few phases. */
		if (diagnostic)
			fprintf (logfp,"Calling init_event\n");
		if (init_event(&e, h) ){
			handle_error();
			free(h); free(p);
			continue;
		}

		/* Rank hypocentres in the order that they will be used as seeds. */
		if (diagnostic)
			fprintf (logfp,"Calling rank_hyp\n");
		if (rank_hyp(&e, h, p) ){
			handle_error();
			free(h); free(p);
			continue;
		}

		/* Print out hypocentres. */
		if (diagnostic)
			fprintf (logfp,"Calling print_hyp\n");
		if ( print_hyp(&e, h) ){
			handle_error();
			free(h); free(p);
			continue;
		}

		/* Option loop */
		/* option 0 is free depth  */
		/* option 1 is fixed depth */
		/* option 2 is fixed depth and location. */
		/* If everything is fixed skip this loop altogether. */
		for (option=0;option<3;option++){

			/* If time is fixed then set solution as required and leave. */
			if (e.fixed_time != NULLVAL){
				if (diagnostic)
					fprintf (logfp,"Calling init_sol to fix all\n");
				if ( init_sol(&s, &e, &h[0]) )
					handle_error();
				s.number_of_unknowns = 0;
				s.converged = 1;
				option=3;
				break;
			}

			/* No free depth pass if fixed depth instruction. */
			if (e.fixed_depth != NULLVAL || e.fix_on_depdp)
				option++;

			/* If location is fixed then depth will always be fixed. */
			if (e.fixed_lat != NULLVAL)
				option++;

			if (diagnostic)
				fprintf(logfp,"option %d\n",option);

			if (option==0)
				s.number_of_unknowns = 4;
			else if (option==1)
				s.number_of_unknowns = 3;
			else
				s.number_of_unknowns = 1;

			/* Seed loop */
			hyp=0;
			while (hyp<e.numhyps){

				/* Avoid ISC seed while testing */
				if (strcmp(h[hyp].agency,"ISC")==0){
					hyp++;
					continue;
				}

				/* Set up the solution structure using the current seed. */
				if (diagnostic)
					fprintf (logfp,"Calling init_sol\n");
				if ( init_sol(&s, &e, &h[hyp]) ){
					handle_error();
					hyp++;
					continue;
				}

				/* New seed so reset purged flag on phases. */
				for (i=0; i<s.numphas; i++){
					p[i].purged=0;
				}

				/* Iteration loop */
				for (s.iteration=0; s.iteration<max_iter; s.iteration++){

					fprintf(logfp,"opt%d %s ",option,h[hyp].agency);
					fprintf(logfp,"iter %d: ", s.iteration);
					print_sol(&s);

					/* Set delta and seaz for each phase. */
					if (diagnostic)
						fprintf (logfp,"Calling calc_delaz\n");
					if ( calc_delaz(&s,p) ){
						handle_error();
						break;
					}

					/* Set ISC phase id for each phase. */
					if (diagnostic)
						fprintf (logfp,"Calling id_pha\n");
					if ( id_pha(&s,p,e.reid_phase) ){
						handle_error();
						break;
					}

					/* If want to fix on depdp then calculate it here.	*/
					if (e.fix_on_depdp){
						if (diagnostic)
							fprintf (logfp,"Calling calc_depdp\n");
						if (calc_depdp(&s, p)){
							handle_error();
							break;
						}
						if (s.depdp == NULLVAL){
							sprintf(errstr,"no depdp to fix on");
							handle_error();
							break;
						}
						s.depth = s.depdp;
						fprintf (logfp,"Fixing depth on depdp=%f\n",s.depdp);

						/* Redo phase id in case some no longer right. */
						if (diagnostic)
							fprintf (logfp,"Calling id_pha\n");
						if ( id_pha(&s,p,e.reid_phase) ){
							handle_error();
							break;
						}
					}

					/* Set weight_factor for each phase. */
					if (diagnostic)
						fprintf (logfp,"Calling get_weight_factor\n");
					if ( get_weight_factor(p,s.numphas) ){
						handle_error();
						break;
					}

					/* Deal with duplicate readings. */
					if (diagnostic)
						fprintf (logfp,"Calling mark_duplicates\n");
					if ( mark_duplicates(&s, p) ){
						handle_error();
						break;
					}

					/* Set ttime, resid, dtdh, and dtdd for each phase. */
					if (diagnostic)
						fprintf (logfp,"Calling calc_resid\n");
					if ( calc_resid(&s,p,"use") ){
						handle_error();
						break;
					}

					/* Set weight for each phase. */
					if (diagnostic)
						fprintf (logfp,"Calling calc_weight\n");
					if ( calc_weight(&s, p) ){
						handle_error();
						break;
					}

					/* Solve equations and update solution structure. */
					if (diagnostic)
						fprintf (logfp,"Calling solve\n");
					if (solve(&s, p)){
						handle_error();
						break;
					}

					/* Force at least 2 iterations. */
					if (s.iteration == 0)
						continue;	

					/* Check for convergence/divergence. */
					/* Update solution if needed. */
					if (diagnostic)
						fprintf (logfp,"Calling decide_iter\n");
					if (decide_iter(&s, p)){
						handle_error();
						break;
					}

					if (s.diverging){
						fprintf(logfp,"DIVERGING\n");
						break;
					}

					/* After first convergence remove phases with large		*/
					/* residuals and change weighting type.					*/
					/* Either of these facilities can be switched off.		*/
					if (s.converged){

						fprintf(logfp,"CONVERGED\n");

						/* Change weighting if hasn't already been done	*/
						/* and if a second weighting_type is given.		*/
						/* Sets converged to 0 if it changes weighting. */
						if (diagnostic)
							fprintf (logfp,"Calling change_weighting\n");
						if (change_weighting(&s)){
							handle_error();
							s.diverging = 1;
							break;
						}

						/* Remove phases with large resids from		*/
						/* solution. Only does this once.			*/
						/* Sets converged to 0 if it purges phases. */
						if (diagnostic)
							fprintf (logfp,"Calling purge_pha\n");
						if (purge_pha(&e, &s, p)){
							handle_error();
							s.diverging = 1;
							break;
						}

						/* Iterate again if something has been done.		*/
						/* Set iteration to -1 to make new solution start	*/
						/* at 0 (iteration++ in for) This ensures that		*/
						/* weights get re-initialised in calc_weight.		*/
						if (!s.converged){
							s.iteration=-1;
						}
					}

					/* If solution found then stop iterating. */
					if (s.converged)
						break;
				}										 /* Iteration loop */

				fprintf(logfp,"\nlast  : ");
				print_sol(&s);
				fprintf(logfp,"\n");

				/* Unless convergence has been	*/
				/* reached try another seed.	*/
				if (s.converged)
					break;

				hyp++;
			}										/* Seed loop */

			/* Don't need another option if converged already. */
			if (s.converged)
				break;
		}										/* Option loop */

		/* Could either be here because of success or failure. */
		if (s.converged){

			/* Calculate residuals for final solution. */

			/* Set delta and seaz for each phase. */
			if (diagnostic)
				fprintf (logfp,"Calling calc_delaz\n");
				if ( calc_delaz(&s,p) ){
				handle_error();
				break;
			}

			/* Calculate residuals. */
			if (diagnostic)
				fprintf (logfp,"Calling calc_resid\n");
			if ( calc_resid(&s,p,"all") )
				handle_error();

			/* Calculate errors etc for final solution. */
			if (diagnostic)
				fprintf (logfp,"Calling calc_error\n");
			if (calc_error(&s,p))
				handle_error();

			/* Calculate magnitudes for final solution. */
			if (diagnostic)
				fprintf (logfp,"Calling calc_netmag\n");
			if (calc_netmag(&s,p))
				handle_error();

			/* Calculate depth phase depth if possible. */
			if (diagnostic)
				fprintf (logfp,"Calling calc_depdp\n");
			if (calc_depdp(&s, p))
				handle_error();

			/* Print phases and residuals to log file. */
			print_pha(s.numphas,p);

			fprintf(logfp,"\nfinal : ");
			print_sol(&s);
			if (s.depdp != NULLVAL)
				fprintf(logfp,"depdp=%.2f err=%.2f\n",s.depdp,s.depdp_error);
			if (s.bodymag)
				fprintf(logfp,"mb=%.2f nsta=%d\n",s.bodymag,s.nsta_mb);
			if (s.surfmag)
				fprintf(logfp,"ms=%.2f nsta=%d\n",s.surfmag,s.nsta_ms);

			/* Write solution to database if wanted. */
			/* Gets new hypid from sequence. */
			if (update_db && database){
				#ifndef NO_ORACLE
					if (diagnostic)
						fprintf (logfp,"Calling put_data\n");
					if (put_data(&e, &s, p))
						handle_error();
				#else
					fprintf (logfp,"ORACLE database not present\n");
				#endif
			}
			/* Will be no isc hypid if db write not done. Need one for ISF. */
			else{
				if (update_db){
					sprintf(errstr,"put_data: no database connection");
					handle_error();
				}
				s.hypid = total;
			}

			/* Write event to ISF file if required. */
			if (isf_outfile[0])
				if (write_isf_event(isfout, &e, &s, h, p))
					handle_error();

			/* Update counters. */
			if (option==0) opt0++;
			if (option==1) opt1++;
			if (option==2) opt2++;
			if (option<3) total++;
		}
		else{
			/* Reset solution to favourite seed. */
			if (diagnostic)
				fprintf (logfp,"Calling init_sol\n");
			if ( init_sol(&s, &e, &h[0]) )
				handle_error();
			fprintf(logfp,"FAILURE\n");
			fail++;
		}

		/* Free the arrays used for hypocentres and phases. */
		free(h);
		free(p);
	}										/* Event loop */

	fprintf(logfp,"\nopt0 %d, opt1 %d, opt2 %d ",opt0,opt1,opt2);
	fprintf(logfp,"converge %d fail %d\n",total,fail);

	close(logfp);
	if (isf_infile[0])
		close(isfin);
	if (isf_outfile[0])
		close(isfout);

	#ifndef NO_ORACLE
		if (database)
			oracle_disconn();
	#endif

	exit(0);
}


