#include "location.h"

/*#DOC  Title:																*/
/*#DOC    pP_duplicates														*/
/*#DOC  Desc: If there is more than one pP phase for a given station		*/
/*#DOC    	  this function will weight each of them down.					*/
/*#DOC  Input Arguments:													*/
/*#DOC    sp - pointer to structure containing current solution.			*/
/*#DOC    p  - array of pointers to structures containing phases.			*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/


int pP_duplicates(struct sol_rec* sp, struct pha_rec p[])
{
	int samesta[PHA_PER_READ];
	char prevnet[NETLEN];
	char prevsta[STALEN];
	int i,j,num_pP;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* Create an array samesta of pP phase indexes for each station */
	num_pP = 0;
	strcpy(prevsta,"");
	strcpy(prevnet,"");
	for (i=0; i<sp->numphas; i++){

		if (strcmp(p[i].phase,"pP"))
			continue;

		/* New station - finish last station and reset things. */
		if (strcmp(p[i].sta,prevsta) || strcmp(p[i].net,prevnet)){

			/* Weight down all pP phases if there's more than one. */
			for (j=0; j<num_pP; j++){
				p[samesta[j]].weight_factor = 1.0/num_pP;

				if (diagnostic){
					fprintf(logfp,"pP_duplicates: %s ",p[samesta[j]].sta);
					fprintf(logfp,"%f\n",p[samesta[j]].weight_factor);
				}
			}

			strcpy(prevsta,p[i].sta);
			strcpy(prevnet,p[i].net);
			num_pP = 0;
		}
		samesta[num_pP++] = i;

		/* Check not going past end of samesta array. */
		if (j>PHA_PER_READ){
			sprintf(errstr,"pP_duplicates: %s: too many pP phases",p[i].sta);
			return 1;
		}
	}
	/* Last station. */
	/* Weight down all pP phases if ther's more than one. */
	for (j=0; j<num_pP; j++){
		p[samesta[j]].weight_factor = 1.0/num_pP;

		if (diagnostic){
			fprintf(logfp,"pP_duplicates: %s ",p[samesta[j]].sta);
			fprintf(logfp,"%f\n",p[samesta[j]].weight_factor);
		}
	}
	return 0;
}
