#include "location.h"

/*#DOC  Title:																*/
/*#DOC    print_hyp															*/
/*#DOC  Desc:																*/
/*#DOC    Writes a table of hypocentres for an event to the log.			*/
/*#DOC  Input Arguments:													*/
/*#DOC    ep  - pointer to structure containing event information.			*/
/*#DOC    h[] - array hypocentre structures.								*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/
/*#DOC  Calls:																*/
/*#DOC	  write_time() from utils.c.										*/

int print_hyp(struct event_rec *ep, struct hyp_rec *h)
{
	char timestr[24];
	int i;

	fprintf(logfp,"evid=%d\n",ep->evid);
	fprintf(logfp,"numhyps=%d numphas=%d\n",ep->numhyps,ep->numphas);
	fprintf(logfp,"hypid   agency time                    lat     lon      ");
	fprintf(logfp,"depth  nsta  def ndef nass gap   mindist ");
	fprintf(logfp,"stime sdep  smaj  score\n");

	for (i=0; i<ep->numhyps; i++){

		fprintf(logfp,"%7d ",h[i].hypid);
		fprintf(logfp,"%-6s %23s ",h[i].agency,write_time(h[i].time,timestr));
		fprintf(logfp,"%7.3f %8.3f ",h[i].lat,h[i].lon);
		if (h[i].depth != NULLVAL)
			fprintf(logfp,"%5.1f  ",h[i].depth);
		else
			fprintf(logfp,"%5s  ","");
		if (h[i].nsta != NULLVAL)
			fprintf(logfp,"%4d ",h[i].nsta);
		else
			fprintf(logfp,"%4s ","");
		if (h[i].ndefsta != NULLVAL)
			fprintf(logfp,"%4d ",h[i].ndefsta);
		else
			fprintf(logfp,"%4s ","");
		if (h[i].ndef != NULLVAL)
			fprintf(logfp,"%4d ",h[i].ndef);
		else
			fprintf(logfp,"%4s ","");
		if (h[i].nass != NULLVAL)
			fprintf(logfp,"%4d ",h[i].nass);
		else
			fprintf(logfp,"%4s ","");
		if (h[i].azimgap != NULLVAL)
			fprintf(logfp,"%5.1f ",h[i].azimgap);
		else
			fprintf(logfp,"%5s ","");
		if (h[i].mindist != NULLVAL)
			fprintf(logfp,"%6.2f ",h[i].mindist);
		else
			fprintf(logfp,"%6s ","");
		if (h[i].stime != NULLVAL)
			fprintf(logfp,"%5.2f ",h[i].stime);
		else
			fprintf(logfp,"%5s ","");
		if (h[i].sdepth != NULLVAL)
			fprintf(logfp,"%5.2f ",h[i].sdepth);
		else
			fprintf(logfp,"%5s ","");
		if (h[i].majax != NULLVAL)
			fprintf(logfp,"%5.1f ",h[i].majax);
		else
			fprintf(logfp,"%5s ","");

		fprintf(logfp,"%5d ",h[i].rank);
		fprintf(logfp,"\n");
	}
	fprintf(logfp,"\n");

	return 0;
}

