#include "location.h"

/*#DOC  Title:																*/
/*#DOC    rank_hyp															*/
/*#DOC  Desc:																*/
/*#DOC    Ranks hypocentres in the order they will be used as seeds.		*/
/*#DOC  Input Arguments:													*/
/*#DOC    ep  - pointer to structure containing event information.			*/
/*#DOC    h - array structures containing hypocentres.						*/
/*#DOC    p - array structures containing phases.							*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/
/*#DOC  Uses:																*/
/*#DOC    calc_gap() from calc_gap.c.										*/

int rank_hyp(struct event_rec *ep, struct hyp_rec h[], struct pha_rec p[])
{
	int i,j;
	int score;
	int number,gap,temp;
	struct hyp_rec h_temp;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* Calculate azimuthal gap, mindist, and nsta for ranking. */
	if (diagnostic)
		fprintf(logfp,"Calling calc_gap\n");
	if ( calc_gap(ep, h, p) )
		return 0;

	/* Loop over hypocentres in this event. */
	for (i=0; i < ep->numhyps; i++){
		score = 0;

		/* Always use ISC hypocentre as seed unless editor says otherwise. */
		if (strcmp(ep->seed_agency,"")){
			if (strcmp(h[i].agency,ep->seed_agency) == 0)
				score += 100;
		}
		else if (strcmp(h[i].agency,"ISC") == 0)
			score += 100;

		if (diagnostic)
			fprintf(logfp,"hyp %d from %s gets %d ",i,h[i].agency,score);

		/* Get best guess of the number of phases they used */
		if (h[i].ndef != NULLVAL)
			number = h[i].ndef;
		else if (h[i].ndefsta != NULLVAL)
			number = h[i].ndefsta;
		else if (h[i].nsta != NULLVAL)
			number = h[i].nsta;
		else
			number=1;

		score += log10(number);			/* 0 - 2 (or 3 for 1000+) */

		if (diagnostic)
			fprintf(logfp,"+ %d ",(int)log10(number));

		/* Azimuthal gap */
		if (h[i].azimgap != NULLVAL)
			gap = h[i].azimgap;
		else
			gap = 360;

		score += (360-gap)/60; 			/* 0 - 5 (or 6 for gap=0) */

		if (diagnostic)
			fprintf(logfp,"+ %d ",(int)((360-gap)/60));

		/* Only consider mindist < 5 */
		/* Otherwise too complicated, eg 20 better than 10 */

		if ( h[i].mindist < 5 ){
			score += 3;
			if (diagnostic)
				fprintf(logfp,"+ 3 ");
		}
		h[i].rank = score;

		if (diagnostic)
			fprintf(logfp,"= %d points\n",score);
	}
	
	/* Sort into score order */
	for(i=0; i< ep->numhyps; i++){
		for (j=i-1; j>-1; j--){

			if (h[j].rank < h[j+1].rank){

				h_temp = h[j+1];
				h[j+1] = h[j];
				h[j]   = h_temp;
			}
		}
	}
	return 0;
}
