#include "location.h"

/*#DOC  Title:																*/
/*#DOC    read_config														*/
/*#DOC  Desc:																*/
/*#DOC    Sets external variables to values read from configuration file.	*/
/*#DOC  Input Arguments:													*/
/*#DOC    filename  - string with path/name of configuration file.			*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/

/* Each line can have one configuration variable and its value, eg:			*/
/*	max_iter = 16															*/
/* Empty lines are ignored.													*/
/* Anything after # is considered a comment and not read.					*/

/* Diagnostics from this function get written to STDERR as logfile is not	*/
/* open yet (its name is read in here).										*/

/* These external variables correspond to recognised entries in the file.	*/ 
char weighting1[VALLEN];
char weighting2[VALLEN];
char out_agency[VALLEN];
int min_phases;
double init_max_resid;
double max_resid;
double purge_resid;
double sigma_start;
double mu;
int max_iter;
double default_depth;
double max_err_factor;
double avg_weight_thresh;
double alpha_thresh;
double dalpha_thresh;
double dsigma_low;
double dsigma_high;
int crust_corr_on;
int elev_corr_on;
int repid;
char isf_stafile[VALLEN];
char isf_outfile[VALLEN];
char instructfile[VALLEN];
char logfile[VALLEN];
char errfile[VALLEN];
double update_db;
double body_mag_min_dist;
double body_mag_max_dist;
double surf_mag_min_dist;
double surf_mag_max_dist;
double body_mag_min_per;
double body_mag_max_per;
double surf_mag_min_per;
double surf_mag_max_per;
double mag_range_warn_thresh;
double depdp_range_warn_thresh;

int read_config(char *filename)
{
	FILE *fp;
	char line[LINLEN];
	char par[PARLEN];
	char value[VALLEN];

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* Open configuration file or return an error. */
	if ((fp = fopen(filename,"r")) == NULL){
		sprintf(errstr,"read_config: can't open %s",filename);
		return 1;
	}

	/* Step through file a line at a time identifying parameters. */
	while ( fgets(line,LINLEN,fp) ){

		/* blank line */
		if ( sscanf(line,"%s = %s", par, value) < 2)
			continue;

		/* comment */
		if (strncmp(par,"#",1)==0)
			continue;

		else if (strcmp(par,"out_agency")==0){
			strncpy(out_agency,value,VALLEN);
			out_agency[VALLEN-1] = '\0';
		}
		else if (strcmp(par,"weighting1")==0){
			strncpy(weighting1,value,VALLEN);
			weighting1[VALLEN-1] = '\0';
		}
		else if (strcmp(par,"weighting2")==0){
			strncpy(weighting2,value,VALLEN);
			weighting2[VALLEN-1] = '\0';
		}
		else if (strcmp(par,"min_phases")==0)
			min_phases = atoi(value);

		else if (strcmp(par,"init_max_resid")==0)
			init_max_resid = atof(value);

		else if (strcmp(par,"max_resid")==0)
			max_resid = atof(value);

		else if (strcmp(par,"purge_resid")==0)
			purge_resid = atof(value);

		else if (strcmp(par,"sigma_start")==0)
			sigma_start = atof(value);

		else if (strcmp(par,"mu")==0)
			mu = atof(value);

		else if (strcmp(par,"max_iter")==0)
			max_iter = atoi(value);

		else if (strcmp(par,"default_depth")==0)
			default_depth = atof(value);

		else if (strcmp(par,"max_err_factor")==0)
			max_err_factor = atof(value);

		else if (strcmp(par,"avg_weight_thresh")==0)
			avg_weight_thresh = atof(value);

		else if (strcmp(par,"alpha_thresh")==0)
			alpha_thresh = atof(value);

		else if (strcmp(par,"dalpha_thresh")==0)
			dalpha_thresh = atof(value);

		else if (strcmp(par,"dsigma_low")==0)
			dsigma_low = atof(value);

		else if (strcmp(par,"dsigma_high")==0)
			dsigma_high = atof(value);

		else if (strcmp(par,"crust_corr_on")==0)
			crust_corr_on = atoi(value);

		else if (strcmp(par,"elev_corr_on")==0)
			elev_corr_on = atoi(value);

		else if (strcmp(par,"repid")==0)
			repid = atoi(value);

		else if (strcmp(par,"isf_stafile")==0){
			strncpy(isf_stafile,value,VALLEN);
			isf_stafile[VALLEN-1] = '\0';
		}
		else if (strcmp(par,"isf_outfile")==0){
			strncpy(isf_outfile,value,VALLEN);
			isf_outfile[VALLEN-1] = '\0';
		}
		else if (strcmp(par,"instructfile")==0){
			strncpy(instructfile,value,VALLEN);
			instructfile[VALLEN-1] = '\0';
		}
		else if (strcmp(par,"logfile")==0){
			strncpy(logfile,value,VALLEN);
			logfile[VALLEN-1] = '\0';
		}
		else if (strcmp(par,"errfile")==0){
			strncpy(errfile,value,VALLEN);
			errfile[VALLEN-1] = '\0';
		}
		else if (strcmp(par,"update_db")==0)
			update_db = atoi(value);

		else if (strcmp(par,"body_mag_min_dist")==0)
			body_mag_min_dist = atof(value);

		else if (strcmp(par,"body_mag_max_dist")==0)
			body_mag_max_dist = atof(value);

		else if (strcmp(par,"surf_mag_min_dist")==0)
			surf_mag_min_dist = atof(value);

		else if (strcmp(par,"surf_mag_max_dist")==0)
			surf_mag_max_dist = atof(value);

		else if (strcmp(par,"body_mag_min_per")==0)
			body_mag_min_per = atof(value);

		else if (strcmp(par,"body_mag_max_per")==0)
			body_mag_max_per = atof(value);

		else if (strcmp(par,"surf_mag_min_per")==0)
			surf_mag_min_per = atof(value);

		else if (strcmp(par,"surf_mag_max_per")==0)
			surf_mag_max_per = atof(value);

		else if (strcmp(par,"mag_range_warn_thresh")==0)
			mag_range_warn_thresh = atof(value);

		else if (strcmp(par,"depdp_range_warn_thresh")==0)
			depdp_range_warn_thresh = atof(value);

		else{
			sprintf(errstr,"read_config: unrecognised parameter %s",par);
			return 1;
		}

		/* logfile not open yet. */
		if (diagnostic)
			fprintf(stderr,"%-*s = %-*s\n",PARLEN,par,VALLEN,value);
	}
	fclose(fp);
	return 0;
}
