#include "location.h"
#include "isf_bul.h"

struct sta_rec {
	char net[NETLEN];
	char sta[STALEN];
	double lat;
	double lon;
	double elev;
};

int read_sta(char *line, char *net, char *sta, char *type, float *lat,
				float *lon, char *coord, float *elev, char *start, char *stop);

/*#DOC  Title:																*/
/*#DOC    read_stafile														*/
/*#DOC  Desc:																*/
/*#DOC    Reads stations from ISF stafile into an array.					*/
/*#DOC  Input Arguments:													*/
/*#DOC    stalist[]  - array of structures for station information.			*/
/*#DOC  Returns:															*/
/*#DOC    number of stations read.											*/
/*#DOC  Uses:																*/
/*#DOC	  read_sta from isf_sta.c.											*/

int read_stafile(struct sta_rec stalist[])
{
	extern char isf_stafile[];		/* From config file */

	FILE *fp;
    char line[ISF_LINE_LEN];
    char net[ISF_NET_LEN+1];
    char sta[ISF_STA_LEN+1];
	float lat,lon,elev;
	int i;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;


	/* Open file for reading in stations. */
	if ((fp = fopen(isf_stafile,"r")) == NULL){
		fprintf(stderr,"read_stafile: Can't open %s\n",isf_stafile);
		return 1;
    }

	i=0;
	while (fgets(line,ISF_LINE_LEN*2,fp)){

		if (read_sta(line,net,sta,"",&lat,&lon,"",&elev,"","")){
			sprintf(errstr,"read_sta: %s",isf_bulletin_error);
			return 1;
		}

		strcpy(stalist[i].net,net);
		strcpy(stalist[i].sta,sta);
		stalist[i].lat = lat;
		stalist[i].lon = lon;

		if (elev == ISF_NULL)
			stalist[i].elev = NULLVAL;
		else
			stalist[i].elev = elev;
		i++;
	}
	fclose(fp);
	return i;
}
